@@ -1,8 +1,11 @@
-from __future__ import absolute_import, division, with_statement
+from __future__ import absolute_import, division, print_function, with_statement
 
+import datetime
 import os
 import tornado.locale
-import unittest
+from tornado.escape import utf8
+from tornado.test.util import unittest
+from tornado.util import u, unicode_type
 
 
 class TranslationLoaderTest(unittest.TestCase):
@@ -29,7 +32,7 @@ class TranslationLoaderTest(unittest.TestCase):
             os.path.join(os.path.dirname(__file__), 'csv_translations'))
         locale = tornado.locale.get("fr_FR")
         self.assertTrue(isinstance(locale, tornado.locale.CSVLocale))
-        self.assertEqual(locale.translate("school"), u"\u00e9cole")
+        self.assertEqual(locale.translate("school"), u("\u00e9cole"))
 
     def test_gettext(self):
         tornado.locale.load_gettext_translations(
@@ -37,4 +40,20 @@ class TranslationLoaderTest(unittest.TestCase):
             "tornado_test")
         locale = tornado.locale.get("fr_FR")
         self.assertTrue(isinstance(locale, tornado.locale.GettextLocale))
-        self.assertEqual(locale.translate("school"), u"\u00e9cole")
+        self.assertEqual(locale.translate("school"), u("\u00e9cole"))
+
+
+class LocaleDataTest(unittest.TestCase):
+    def test_non_ascii_name(self):
+        name = tornado.locale.LOCALE_NAMES['es_LA']['name']
+        self.assertTrue(isinstance(name, unicode_type))
+        self.assertEqual(name, u('Espa\u00f1ol'))
+        self.assertEqual(utf8(name), b'Espa\xc3\xb1ol')
+
+
+class EnglishTest(unittest.TestCase):
+    def test_format_date(self):
+        locale = tornado.locale.get('en_US')
+        date = datetime.datetime(2013, 4, 28, 18, 35)
+        self.assertEqual(locale.format_date(date, full_format=True),
+                         'April 28, 2013 at 6:35 pm')
